package net.mcreator.geafm.network;

import net.neoforged.neoforge.registries.NeoForgeRegistries;
import net.neoforged.neoforge.registries.DeferredRegister;
import net.neoforged.neoforge.network.handling.PlayPayloadContext;
import net.neoforged.neoforge.network.PacketDistributor;
import net.neoforged.neoforge.event.entity.player.PlayerEvent;
import net.neoforged.neoforge.common.util.INBTSerializable;
import net.neoforged.neoforge.attachment.AttachmentType;
import net.neoforged.fml.event.lifecycle.FMLCommonSetupEvent;
import net.neoforged.fml.common.Mod;
import net.neoforged.bus.api.SubscribeEvent;

import net.minecraft.world.level.saveddata.SavedData;
import net.minecraft.world.level.ServerLevelAccessor;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.Entity;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.network.protocol.common.custom.CustomPacketPayload;
import net.minecraft.network.protocol.PacketFlow;
import net.minecraft.network.chat.Component;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.client.Minecraft;

import net.mcreator.geafm.GeafmMod;

import java.util.function.Supplier;

@Mod.EventBusSubscriber(bus = Mod.EventBusSubscriber.Bus.MOD)
public class GeafmModVariables {
	public static final DeferredRegister<AttachmentType<?>> ATTACHMENT_TYPES = DeferredRegister.create(NeoForgeRegistries.Keys.ATTACHMENT_TYPES, GeafmMod.MODID);
	public static final Supplier<AttachmentType<PlayerVariables>> PLAYER_VARIABLES = ATTACHMENT_TYPES.register("player_variables", () -> AttachmentType.serializable(() -> new PlayerVariables()).build());

	@SubscribeEvent
	public static void init(FMLCommonSetupEvent event) {
		GeafmMod.addNetworkMessage(SavedDataSyncMessage.ID, SavedDataSyncMessage::new, SavedDataSyncMessage::handleData);
		GeafmMod.addNetworkMessage(PlayerVariablesSyncMessage.ID, PlayerVariablesSyncMessage::new, PlayerVariablesSyncMessage::handleData);
	}

	@Mod.EventBusSubscriber
	public static class EventBusVariableHandlers {
		@SubscribeEvent
		public static void onPlayerLoggedInSyncPlayerVariables(PlayerEvent.PlayerLoggedInEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void onPlayerRespawnedSyncPlayerVariables(PlayerEvent.PlayerRespawnEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void onPlayerChangedDimensionSyncPlayerVariables(PlayerEvent.PlayerChangedDimensionEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void clonePlayer(PlayerEvent.Clone event) {
			PlayerVariables original = event.getOriginal().getData(PLAYER_VARIABLES);
			PlayerVariables clone = new PlayerVariables();
			clone.DragonQuestDone = original.DragonQuestDone;
			clone.FlightQuestDone = original.FlightQuestDone;
			clone.EnderDragonDefeated = original.EnderDragonDefeated;
			clone.WitherDefeated = original.WitherDefeated;
			clone.TimesAscended = original.TimesAscended;
			clone.HasHadAntimatterBar = original.HasHadAntimatterBar;
			clone.HasHadGodBar = original.HasHadGodBar;
			clone.IsKeepInventoryOn = original.IsKeepInventoryOn;
			clone.GodBlood = original.GodBlood;
			clone.IsInAscensionDimension = original.IsInAscensionDimension;
			clone.GodXP = original.GodXP;
			clone.FreeAscensionBought = original.FreeAscensionBought;
			clone.CheaperUpgrades = original.CheaperUpgrades;
			clone.hasHadStarCraftingTableBook = original.hasHadStarCraftingTableBook;
			clone.hasHadMaterialCompressorBook = original.hasHadMaterialCompressorBook;
			if (!event.isWasDeath()) {
				clone.pageNumber = original.pageNumber;
				clone.recipe1 = original.recipe1;
				clone.recipe2 = original.recipe2;
			}
			event.getEntity().setData(PLAYER_VARIABLES, clone);
		}

		@SubscribeEvent
		public static void onPlayerLoggedIn(PlayerEvent.PlayerLoggedInEvent event) {
			if (event.getEntity() instanceof ServerPlayer player) {
				SavedData mapdata = MapVariables.get(event.getEntity().level());
				SavedData worlddata = WorldVariables.get(event.getEntity().level());
				if (mapdata != null)
					PacketDistributor.PLAYER.with(player).send(new SavedDataSyncMessage(0, mapdata));
				if (worlddata != null)
					PacketDistributor.PLAYER.with(player).send(new SavedDataSyncMessage(1, worlddata));
			}
		}

		@SubscribeEvent
		public static void onPlayerChangedDimension(PlayerEvent.PlayerChangedDimensionEvent event) {
			if (event.getEntity() instanceof ServerPlayer player) {
				SavedData worlddata = WorldVariables.get(event.getEntity().level());
				if (worlddata != null)
					PacketDistributor.PLAYER.with(player).send(new SavedDataSyncMessage(1, worlddata));
			}
		}
	}

	public static class WorldVariables extends SavedData {
		public static final String DATA_NAME = "geafm_worldvars";

		public static WorldVariables load(CompoundTag tag) {
			WorldVariables data = new WorldVariables();
			data.read(tag);
			return data;
		}

		public void read(CompoundTag nbt) {
		}

		@Override
		public CompoundTag save(CompoundTag nbt) {
			return nbt;
		}

		public void syncData(LevelAccessor world) {
			this.setDirty();
			if (world instanceof Level level && !level.isClientSide())
				PacketDistributor.DIMENSION.with(level.dimension()).send(new SavedDataSyncMessage(1, this));
		}

		static WorldVariables clientSide = new WorldVariables();

		public static WorldVariables get(LevelAccessor world) {
			if (world instanceof ServerLevel level) {
				return level.getDataStorage().computeIfAbsent(new SavedData.Factory<>(WorldVariables::new, WorldVariables::load), DATA_NAME);
			} else {
				return clientSide;
			}
		}
	}

	public static class MapVariables extends SavedData {
		public static final String DATA_NAME = "geafm_mapvars";
		public double forgeFlow = 0;

		public static MapVariables load(CompoundTag tag) {
			MapVariables data = new MapVariables();
			data.read(tag);
			return data;
		}

		public void read(CompoundTag nbt) {
			forgeFlow = nbt.getDouble("forgeFlow");
		}

		@Override
		public CompoundTag save(CompoundTag nbt) {
			nbt.putDouble("forgeFlow", forgeFlow);
			return nbt;
		}

		public void syncData(LevelAccessor world) {
			this.setDirty();
			if (world instanceof Level && !world.isClientSide())
				PacketDistributor.ALL.noArg().send(new SavedDataSyncMessage(0, this));
		}

		static MapVariables clientSide = new MapVariables();

		public static MapVariables get(LevelAccessor world) {
			if (world instanceof ServerLevelAccessor serverLevelAcc) {
				return serverLevelAcc.getLevel().getServer().getLevel(Level.OVERWORLD).getDataStorage().computeIfAbsent(new SavedData.Factory<>(MapVariables::new, MapVariables::load), DATA_NAME);
			} else {
				return clientSide;
			}
		}
	}

	public static class SavedDataSyncMessage implements CustomPacketPayload {
		public static final ResourceLocation ID = new ResourceLocation(GeafmMod.MODID, "saved_data_sync");
		private final int type;
		private SavedData data;

		public SavedDataSyncMessage(FriendlyByteBuf buffer) {
			this.type = buffer.readInt();
			CompoundTag nbt = buffer.readNbt();
			if (nbt != null) {
				this.data = this.type == 0 ? new MapVariables() : new WorldVariables();
				if (this.data instanceof MapVariables mapVariables)
					mapVariables.read(nbt);
				else if (this.data instanceof WorldVariables worldVariables)
					worldVariables.read(nbt);
			}
		}

		public SavedDataSyncMessage(int type, SavedData data) {
			this.type = type;
			this.data = data;
		}

		@Override
		public void write(final FriendlyByteBuf buffer) {
			buffer.writeInt(type);
			if (data != null)
				buffer.writeNbt(data.save(new CompoundTag()));
		}

		@Override
		public ResourceLocation id() {
			return ID;
		}

		public static void handleData(final SavedDataSyncMessage message, final PlayPayloadContext context) {
			if (context.flow() == PacketFlow.CLIENTBOUND && message.data != null) {
				context.workHandler().submitAsync(() -> {
					if (message.type == 0)
						MapVariables.clientSide.read(message.data.save(new CompoundTag()));
					else
						WorldVariables.clientSide.read(message.data.save(new CompoundTag()));
				}).exceptionally(e -> {
					context.packetHandler().disconnect(Component.literal(e.getMessage()));
					return null;
				});
			}
		}
	}

	public static class PlayerVariables implements INBTSerializable<CompoundTag> {
		public boolean DragonQuestDone = false;
		public boolean FlightQuestDone = false;
		public boolean EnderDragonDefeated = false;
		public boolean WitherDefeated = false;
		public double TimesAscended = 0;
		public boolean HasHadAntimatterBar = false;
		public boolean HasHadGodBar = false;
		public boolean IsKeepInventoryOn = false;
		public double GodBlood = 0;
		public boolean IsInAscensionDimension = false;
		public double GodXP = 0.0;
		public boolean FreeAscensionBought = false;
		public boolean CheaperUpgrades = false;
		public boolean hasHadStarCraftingTableBook = false;
		public boolean hasHadMaterialCompressorBook = false;
		public double pageNumber = 1.0;
		public String recipe1 = "\"\"";
		public String recipe2 = "\"\"";

		@Override
		public CompoundTag serializeNBT() {
			CompoundTag nbt = new CompoundTag();
			nbt.putBoolean("DragonQuestDone", DragonQuestDone);
			nbt.putBoolean("FlightQuestDone", FlightQuestDone);
			nbt.putBoolean("EnderDragonDefeated", EnderDragonDefeated);
			nbt.putBoolean("WitherDefeated", WitherDefeated);
			nbt.putDouble("TimesAscended", TimesAscended);
			nbt.putBoolean("HasHadAntimatterBar", HasHadAntimatterBar);
			nbt.putBoolean("HasHadGodBar", HasHadGodBar);
			nbt.putBoolean("IsKeepInventoryOn", IsKeepInventoryOn);
			nbt.putDouble("GodBlood", GodBlood);
			nbt.putBoolean("IsInAscensionDimension", IsInAscensionDimension);
			nbt.putDouble("GodXP", GodXP);
			nbt.putBoolean("FreeAscensionBought", FreeAscensionBought);
			nbt.putBoolean("CheaperUpgrades", CheaperUpgrades);
			nbt.putBoolean("hasHadStarCraftingTableBook", hasHadStarCraftingTableBook);
			nbt.putBoolean("hasHadMaterialCompressorBook", hasHadMaterialCompressorBook);
			nbt.putDouble("pageNumber", pageNumber);
			nbt.putString("recipe1", recipe1);
			nbt.putString("recipe2", recipe2);
			return nbt;
		}

		@Override
		public void deserializeNBT(CompoundTag nbt) {
			DragonQuestDone = nbt.getBoolean("DragonQuestDone");
			FlightQuestDone = nbt.getBoolean("FlightQuestDone");
			EnderDragonDefeated = nbt.getBoolean("EnderDragonDefeated");
			WitherDefeated = nbt.getBoolean("WitherDefeated");
			TimesAscended = nbt.getDouble("TimesAscended");
			HasHadAntimatterBar = nbt.getBoolean("HasHadAntimatterBar");
			HasHadGodBar = nbt.getBoolean("HasHadGodBar");
			IsKeepInventoryOn = nbt.getBoolean("IsKeepInventoryOn");
			GodBlood = nbt.getDouble("GodBlood");
			IsInAscensionDimension = nbt.getBoolean("IsInAscensionDimension");
			GodXP = nbt.getDouble("GodXP");
			FreeAscensionBought = nbt.getBoolean("FreeAscensionBought");
			CheaperUpgrades = nbt.getBoolean("CheaperUpgrades");
			hasHadStarCraftingTableBook = nbt.getBoolean("hasHadStarCraftingTableBook");
			hasHadMaterialCompressorBook = nbt.getBoolean("hasHadMaterialCompressorBook");
			pageNumber = nbt.getDouble("pageNumber");
			recipe1 = nbt.getString("recipe1");
			recipe2 = nbt.getString("recipe2");
		}

		public void syncPlayerVariables(Entity entity) {
			if (entity instanceof ServerPlayer serverPlayer)
				PacketDistributor.PLAYER.with(serverPlayer).send(new PlayerVariablesSyncMessage(this));
		}
	}

	public record PlayerVariablesSyncMessage(PlayerVariables data) implements CustomPacketPayload {
		public static final ResourceLocation ID = new ResourceLocation(GeafmMod.MODID, "player_variables_sync");

		public PlayerVariablesSyncMessage(FriendlyByteBuf buffer) {
			this(new PlayerVariables());
			this.data.deserializeNBT(buffer.readNbt());
		}

		@Override
		public void write(final FriendlyByteBuf buffer) {
			buffer.writeNbt(data.serializeNBT());
		}

		@Override
		public ResourceLocation id() {
			return ID;
		}

		public static void handleData(final PlayerVariablesSyncMessage message, final PlayPayloadContext context) {
			if (context.flow() == PacketFlow.CLIENTBOUND && message.data != null) {
				context.workHandler().submitAsync(() -> Minecraft.getInstance().player.getData(PLAYER_VARIABLES).deserializeNBT(message.data.serializeNBT())).exceptionally(e -> {
					context.packetHandler().disconnect(Component.literal(e.getMessage()));
					return null;
				});
			}
		}
	}
}
